@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2006-2016,2018,2021 Con Kolivas
+   Copyright (C) 2006-2016,2018,2021-2022 Con Kolivas
    Copyright (C) 2011 Serge Belyshev
    Copyright (C) 2011 Peter Hyman
    Copyright (C) 1998 Andrew Tridgell
@@ -610,20 +610,22 @@ ssize_t put_fdout(rzip_control *control, void *offset_buf, ssize_t ret)
 	if (unlikely(control->out_ofs + ret > control->out_maxlen)) {
 		/* The data won't fit in a temporary output buffer so we have
 		 * to fall back to temporary files. */
-		print_verbose("Unable to decompress entirely in ram, will use physical files\n");
-		if (unlikely(control->fd_out == -1))
-			failure("Was unable to decompress entirely in ram and no temporary file creation was possible\n");
+		print_verbose("Unable to %scompress entirely in ram, will use physical files\n",
+			      DECOMPRESS ? "de" : "");
+		if (unlikely(control->fd_out == -1)) {
+			failure("Was unable to %scompress entirely in ram and no temporary file creation was possible\n",
+				DECOMPRESS ? "de" : "");
+		}
+		/* Copy tmp_outbuf to tmpoutfile before deallocation */
 		if (unlikely(!write_fdout(control, control->tmp_outbuf, control->out_len))) {
 			print_err("Unable to write_fdout tmpoutbuf in put_fdout\n");
 			return -1;
 		}
+		/* Deallocate now unused tmpoutbuf and unset tmp_outbuf flag */
 		close_tmpoutbuf(control);
-		if (unlikely(!write_fdout(control, offset_buf, ret))) {
-			print_err("Unable to write_fdout offset_buf in put_fdout\n");
-			return -1;
-		}
-		return ret;
+		return write(control->fd_out, offset_buf, (size_t)ret);
 	}
+
 	memcpy(control->tmp_outbuf + control->out_ofs, offset_buf, ret);
 	control->out_ofs += ret;
 	if (likely(control->out_ofs > control->out_len))
@@ -632,8 +634,7 @@ ssize_t put_fdout(rzip_control *control, void *offset_buf, ssize_t ret)
 }
 
 /* This is a custom version of write() which writes in 1GB chunks to avoid
-   the overflows at the >= 2GB mark thanks to 32bit fuckage. This should help
-   even on the rare occasion write() fails to write 1GB as well. */
+   the overflows at the >= 2GB mark thanks to 32bit fuckage. */
 ssize_t write_1g(rzip_control *control, void *buf, i64 len)
 {
 	uchar *offset_buf = buf;
@@ -642,7 +643,10 @@ ssize_t write_1g(rzip_control *control, void *buf, i64 len)
 
 	total = 0;
 	while (len > 0) {
-		ret = MIN(len, one_g);
+		if (BITS32)
+			ret = MIN(len, one_g);
+		else
+			ret = len;
 		ret = put_fdout(control, offset_buf, (size_t)ret);
 		if (unlikely(ret <= 0))
 			return ret;
@@ -693,8 +697,10 @@ ssize_t read_1g(rzip_control *control, int fd, void *buf, i64 len)
 		/* We're decompressing from STDIN */
 		if (unlikely(control->in_ofs + len > control->in_maxlen)) {
 			/* We're unable to fit it all into the temp buffer */
-			if (dump_stdin(control))
-				failure_return(("Inadequate ram to %compress from STDIN and unable to create in tmpfile"), -1);
+			if (dump_stdin(control)) {
+				failure_return(("Inadequate ram to %scompress from STDIN and unable to create in tmpfile",
+					DECOMPRESS ? "de" : ""), -1);
+			}
 			goto read_fd;
 		}
 		if (control->in_ofs + len > control->in_len) {
@@ -717,7 +723,10 @@ ssize_t read_1g(rzip_control *control, int fd, void *buf, i64 len)
 read_fd:
 	total = 0;
 	while (len > 0) {
-		ret = MIN(len, one_g);
+		if (BITS32)
+			ret = MIN(len, one_g);
+		else
+			ret = len;
 		ret = read(fd, offset_buf, (size_t)ret);
 		if (unlikely(ret <= 0))
 			return ret;
@@ -928,9 +937,10 @@ bool close_streamout_threads(rzip_control *control)
 void *open_stream_out(rzip_control *control, int f, unsigned int n, i64 chunk_limit, char cbytes)
 {
 	struct stream_info *sinfo;
+	unsigned int i, testbufs;
+	bool threadlimit = false;
 	i64 testsize, limit;
 	uchar *testmalloc;
-	unsigned int i, testbufs;
 
 	sinfo = calloc(sizeof(struct stream_info), 1);
 	if (unlikely(!sinfo))
@@ -966,20 +976,28 @@ void *open_stream_out(rzip_control *control, int f, unsigned int n, i64 chunk_li
 	/* If we don't have enough ram for the number of threads, decrease the
 	 * number of threads till we do, or only have one thread. */
 	while (limit < STREAM_BUFSIZE && limit < chunk_limit) {
-		if (control->threads > 1)
+		if (control->threads > 1) {
 			--control->threads;
-		else
+			threadlimit = true;
+		} else
 			break;
 		limit = (control->usable_ram - (control->overhead * control->threads)) / testbufs;
 		limit = MIN(limit, chunk_limit);
 	}
+	if (threadlimit) {
+		print_output("Minimising number of threads to %d to limit memory usage\n",
+			     control->threads);
+	}
 	if (BITS32) {
 		limit = MIN(limit, one_g);
 		if (limit + (control->overhead * control->threads) > one_g)
 			limit = one_g - (control->overhead * control->threads);
 	}
 	/* Use a nominal minimum size should we fail all previous shrinking */
-	limit = MAX(limit, STREAM_BUFSIZE);
+	if (limit < STREAM_BUFSIZE) {
+		limit = MAX(limit, STREAM_BUFSIZE);
+		print_output("Warning, low memory for chosen compression settings\n");
+	}
 	limit = MIN(limit, chunk_limit);
 retest_malloc:
 	testsize = limit + (control->overhead * control->threads);
@@ -1361,16 +1379,11 @@ retry:
 	if (!ctis->chunks++) {
 		int j;
 
-		if (TMP_OUTBUF) {
+		if (STDOUT) {
 			lock_mutex(control, &control->control_lock);
 			if (!control->magic_written)
 				write_magic(control);
 			unlock_mutex(control, &control->control_lock);
-
-			if (unlikely(!flush_tmpoutbuf(control))) {
-				print_err("Failed to flush_tmpoutbuf in compthread\n");
-				goto error;
-			}
 		}
 
 		print_maxverbose("Writing initial chunk bytes value %d at %lld\n",
@@ -1673,7 +1686,7 @@ fill_another:
 	fsync(control->fd_out);
 
 	if (unlikely(u_len > control->maxram))
-		print_progress("Warning, attempting to malloc very large buffer for this environment of size %lld\n", u_len);
+		print_output("Warning, attempting to malloc very large buffer for this environment of size %lld\n", u_len);
 	max_len = MAX(u_len, MIN_SIZE);
 	max_len = MAX(max_len, c_len);
 	s_buf = malloc(max_len);
@@ -1849,8 +1862,11 @@ static void add_to_rulist(rzip_control *control, struct stream_info *sinfo)
 		failure("Failed to calloc struct node in add_rulist\n");
 	node->sinfo = sinfo;
 	node->pthreads = control->pthreads;
-	node->prev = control->rulist;
+
+	lock_mutex(control, &control->control_lock);
+	node->prev = control->ruhead;
 	control->ruhead = node;
+	unlock_mutex(control, &control->control_lock);
 }
 
 /* close down an input stream */
@@ -1882,11 +1898,8 @@ int close_stream_in(rzip_control *control, void *ss)
    so do not compress any block that is incompressible by lz4. */
 static int lz4_compresses(rzip_control *control, uchar *s_buf, i64 s_len)
 {
-	int in_len, test_len = s_len, save_len = s_len;
-	int dlen;
+	int dlen, test_len;
 	char *c_buf = NULL, *test_buf = (char *)s_buf;
-	/* set minimum buffer test size based on the length of the test stream */
-	int buftest_size = (test_len > 5 * STREAM_BUFSIZE ? STREAM_BUFSIZE : STREAM_BUFSIZE / 4096);
 	int ret = 0;
 	int workcounter = 0;	/* count # of passes */
 	int best_dlen = INT_MAX; /* save best compression estimate */
@@ -1894,40 +1907,37 @@ static int lz4_compresses(rzip_control *control, uchar *s_buf, i64 s_len)
 	if (!LZ4_TEST)
 		return 1;
 
-	in_len = MIN(test_len, buftest_size);
-	dlen = STREAM_BUFSIZE + STREAM_BUFSIZE / 16 + 64 + 3;
-
+	dlen = MIN(s_len, STREAM_BUFSIZE);
+	test_len = MIN(dlen, STREAM_BUFSIZE >> 8);
 	c_buf = malloc(dlen);
 	if (unlikely(!c_buf))
 		fatal_return(("Unable to allocate c_buf in lz4_compresses\n"), 0);
 
 	/* Test progressively larger blocks at a time and as soon as anything
 	   compressible is found, jump out as a success */
-	while (test_len > 0) {
+	do {
 		int lz4_ret;
 
 		workcounter++;
 		lz4_ret = LZ4_compress_default((const char *)test_buf, c_buf, test_len, dlen);
-		if (!lz4_ret) // Bigger than dlen, no point going further
-			break;
+		if (!lz4_ret) // Bigger than dlen
+			lz4_ret = test_len;
 		if (lz4_ret < best_dlen)
 			best_dlen = lz4_ret;
 		if (lz4_ret < test_len) {
 			ret = 1;
 			break;
 		}
-		/* expand and move buffer */
-		test_len -= in_len;
-		if (test_len) {
-			test_buf += (ptrdiff_t)in_len;
-			if (buftest_size < STREAM_BUFSIZE)
-				buftest_size <<= 1;
-			in_len = MIN(test_len, buftest_size);
-		}
+		/* expand test length */
+		test_len <<= 1;
+	} while (test_len <= dlen);
+
+	if (!ret)
+		print_maxverbose("lz4 testing FAILED for chunk %ld. %d Passes\n", workcounter);
+	else {
+		print_maxverbose("lz4 testing OK for chunk %ld. Compressed size = %5.2F%% of chunk, %d Passes\n",
+				s_len, 100 * ((double) best_dlen / (double) test_len), workcounter);
 	}
-	print_maxverbose("lz4 testing %s for chunk %ld. Compressed size = %5.2F%% of chunk, %d Passes\n",
-			(ret == 0? "FAILED" : "OK"), save_len,
-			100 * ((double) best_dlen / (double) in_len), workcounter);
 
 	dealloc(c_buf);
 
